import type { AgentContext, DidDocument } from '@credo-ts/core';
import { PolygonDID } from '@ayanworks/polygon-did-registrar';
import { SigningKey } from 'ethers';
import { PolygonModuleConfig } from '../PolygonModuleConfig';
interface SchemaRegistryConfig {
    didRegistrarContractAddress: string;
    rpcUrl: string;
    fileServerToken: string;
    privateKey: string;
    schemaManagerContractAddress: string;
    serverUrl: string;
}
export type CreateDidOperationOptions = {
    operation: DidOperation.Create;
    serviceEndpoint?: string;
};
export type UpdateDidOperationOptions = {
    operation: DidOperation.Update;
    didDocument: DidDocument;
    did: string;
};
export type DeactivateDidOperationOptions = {
    operation: DidOperation.Deactivate;
    did: string;
};
export type AddResourceDidOperationOptions = {
    operation: DidOperation.AddResource;
    resourceId: string;
    resource: object;
    did: string;
};
export declare enum DidOperation {
    Create = "createDID",
    Update = "updateDIDDoc",
    Deactivate = "deactivate",
    AddResource = "addResource"
}
export type DidOperationOptions = CreateDidOperationOptions | UpdateDidOperationOptions | DeactivateDidOperationOptions | AddResourceDidOperationOptions;
export type SchemaOperationOptions = CreateSchemaOperationOptions;
export type CreateSchemaOperationOptions = {
    operation: SchemaOperation.CreateSchema;
    did: string;
};
export declare enum SchemaOperation {
    CreateSchema = "createSchema"
}
export declare class PolygonLedgerService {
    rpcUrl: string | undefined;
    private didContractAddress;
    private schemaManagerContractAddress;
    private fileServerToken;
    private fileServerUrl;
    constructor({ didContractAddress, rpcUrl, fileServerToken, schemaManagerContractAddress, serverUrl, }: PolygonModuleConfig);
    createSchema(agentContext: AgentContext, { did, schemaName, schema }: {
        did: string;
        schemaName: string;
        schema: object;
    }): Promise<{
        did: string;
        schemaId: string;
        schemaTxnHash: any;
        resourceTxnHash: any;
        tnxSchemaId?: undefined;
        schemaTxhash?: undefined;
        schemaState?: undefined;
    } | {
        tnxSchemaId: string;
        schemaTxhash: string;
        schemaState: {
            state: string;
            reason: string;
        };
        did?: undefined;
        schemaId?: undefined;
        schemaTxnHash?: undefined;
        resourceTxnHash?: undefined;
    }>;
    getSchemaByDidAndSchemaId(agentContext: AgentContext, did: string, schemaId: string): Promise<any>;
    estimateFeeForDidOperation(agentContext: AgentContext, options: DidOperationOptions): Promise<import("@ayanworks/polygon-did-registrar/build/registrar").EstimatedTxDetails | null | undefined>;
    estimateFeeForSchemaOperation(agentContext: AgentContext, options: SchemaOperationOptions): Promise<{} | undefined>;
    createDidRegistryInstance(signingKey: SigningKey): PolygonDID;
    private createSchemaRegistryInstance;
    private getSigningKey;
    private getPublicKeyFromDid;
    updateModuleConfig({ didRegistrarContractAddress, fileServerToken, rpcUrl, schemaManagerContractAddress, serverUrl, }: SchemaRegistryConfig): void;
}
export {};
