"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PolygonLedgerService = exports.SchemaOperation = exports.DidOperation = void 0;
const polygon_did_registrar_1 = require("@ayanworks/polygon-did-registrar");
const did_1 = require("@ayanworks/polygon-did-registrar/build/utils/did");
const polygon_schema_manager_1 = require("@ayanworks/polygon-schema-manager");
const askar_1 = require("@credo-ts/askar");
const core_1 = require("@credo-ts/core");
const ethers_1 = require("ethers");
const PolygonModuleConfig_1 = require("../PolygonModuleConfig");
const utils_1 = require("../utils");
var DidOperation;
(function (DidOperation) {
    DidOperation["Create"] = "createDID";
    DidOperation["Update"] = "updateDIDDoc";
    DidOperation["Deactivate"] = "deactivate";
    DidOperation["AddResource"] = "addResource";
})(DidOperation = exports.DidOperation || (exports.DidOperation = {}));
var SchemaOperation;
(function (SchemaOperation) {
    SchemaOperation["CreateSchema"] = "createSchema";
})(SchemaOperation = exports.SchemaOperation || (exports.SchemaOperation = {}));
let PolygonLedgerService = class PolygonLedgerService {
    constructor({ didContractAddress, rpcUrl, fileServerToken, schemaManagerContractAddress, serverUrl, }) {
        this.rpcUrl = rpcUrl;
        this.didContractAddress = didContractAddress;
        this.schemaManagerContractAddress = schemaManagerContractAddress;
        this.fileServerToken = fileServerToken;
        this.fileServerUrl = serverUrl;
    }
    async createSchema(agentContext, { did, schemaName, schema }) {
        const publicKeyBase58 = await this.getPublicKeyFromDid(agentContext, did);
        if (!publicKeyBase58) {
            throw new core_1.CredoError('Public Key not found in wallet');
        }
        const signingKey = await this.getSigningKey(agentContext.wallet, publicKeyBase58);
        const schemaRegistry = this.createSchemaRegistryInstance(signingKey);
        agentContext.config.logger.info(`Creating schema on ledger: ${did}`);
        const response = await schemaRegistry.createSchema(did, schemaName, schema);
        if (!response) {
            agentContext.config.logger.error(`Schema creation failed for did: ${did} and schema: ${schema}`);
            throw new core_1.CredoError(`Schema creation failed for did: ${did} and schema: ${schema}`);
        }
        agentContext.config.logger.info(`Published schema on ledger: ${did}`);
        return response;
    }
    async getSchemaByDidAndSchemaId(agentContext, did, schemaId) {
        agentContext.config.logger.info(`Getting schema from ledger: ${did} and schemaId: ${schemaId}`);
        const publicKeyBase58 = await this.getPublicKeyFromDid(agentContext, did);
        if (!publicKeyBase58) {
            throw new core_1.CredoError('Public Key not found in wallet');
        }
        const signingKey = await this.getSigningKey(agentContext.wallet, publicKeyBase58);
        const schemaRegistry = this.createSchemaRegistryInstance(signingKey);
        const response = await schemaRegistry.getSchemaById(did, schemaId);
        if (!response) {
            agentContext.config.logger.error(`Schema not found for did: ${did} and schemaId: ${schemaId} Error: ${response}`);
            throw new core_1.CredoError(`Schema not found for did: ${did} and schemaId: ${schemaId}`);
        }
        agentContext.config.logger.info(`Got schema from ledger: ${did} and schemaId: ${schemaId}`);
        return response;
    }
    async estimateFeeForDidOperation(agentContext, options) {
        const keyPair = await (0, utils_1.generateSecp256k1KeyPair)();
        const didRegistry = this.createDidRegistryInstance(new ethers_1.SigningKey(keyPair.privateKey));
        const { operation } = options;
        if (operation === DidOperation.Create) {
            agentContext.config.logger.info(`Getting estimated fee for operation: ${operation} `);
            const did = `did:polygon:testnet${keyPair.address}`;
            const didDoc = (0, utils_1.getSecp256k1DidDocWithPublicKey)(did, keyPair.publicKeyBase58, options === null || options === void 0 ? void 0 : options.serviceEndpoint);
            const response = await didRegistry.estimateTxFee(DidOperation.Create, [keyPair.address, JSON.stringify(didDoc)]);
            return response;
        }
        if (operation === DidOperation.Update) {
            agentContext.config.logger.info(`Getting estimated fee for operation: ${operation} `);
            const parsedDid = (0, did_1.parseDid)(options.did);
            const response = await didRegistry.estimateTxFee(DidOperation.Update, [parsedDid.didAddress, JSON.stringify(options.didDocument)], parsedDid.didAddress);
            return response;
        }
        if (operation === DidOperation.Deactivate) {
            agentContext.config.logger.info(`Getting estimated fee for operation: ${operation} `);
            const parsedDid = (0, did_1.parseDid)(options.did);
            const deactivatedDidDocument = new core_1.DidDocumentBuilder(options.did)
                .addContext('https://www.w3.org/ns/did/v1')
                .build();
            const response = await didRegistry.estimateTxFee(DidOperation.Update, [parsedDid.didAddress, JSON.stringify(deactivatedDidDocument)], parsedDid.didAddress);
            return response;
        }
        if (operation === DidOperation.AddResource) {
            agentContext.config.logger.info(`Getting estimated fee for operation: ${operation} `);
            const parsedDid = (0, did_1.parseDid)(options.did);
            const response = await didRegistry.estimateTxFee(DidOperation.AddResource, [parsedDid.didAddress, options.resourceId, JSON.stringify(options.resource)], parsedDid.didAddress);
            return response;
        }
    }
    async estimateFeeForSchemaOperation(agentContext, options) {
        const keyPair = await (0, utils_1.generateSecp256k1KeyPair)();
        const schemaRegistry = this.createSchemaRegistryInstance(new ethers_1.SigningKey(keyPair.privateKey));
        const { operation } = options;
        const testResourceBody = {
            resourceURI: 'did:polygon:testnet:0x13cd23928Ae515b86592C630f56C138aE4c7B79a/resources/398cee0a-efac-4643-9f4c-74c48c72a14b',
            resourceCollectionId: '55dbc8bf-fba3-4117-855c-1e0dc1d3bb47',
            resourceId: '398cee0a-efac-4643-9f4c-74c48c72a14b',
            resourceName: 'Eventbrite1 Logo',
            resourceType: 'W3C-schema',
            mediaType: 'image/svg+xml',
            created: '2022-11-17T08:10:36Z',
            checksum: 'a95380f460e63ad939541a57aecbfd795fcd37c6d78ee86c885340e33a91b559',
            previousVersionId: null,
            nextVersionId: null,
        };
        if (operation === SchemaOperation.CreateSchema) {
            agentContext.config.logger.info(`Getting estimated fee for operation: ${operation} `);
            const schemaEstimatedFee = await schemaRegistry.estimateTxFee(SchemaOperation.CreateSchema, [
                keyPair.address,
                core_1.utils.uuid(),
                JSON.stringify(testResourceBody),
            ]);
            const resourceEstimatedFee = await this.estimateFeeForDidOperation(agentContext, {
                operation: DidOperation.AddResource,
                resourceId: core_1.utils.uuid(),
                resource: testResourceBody,
                did: options.did,
            });
            let feeParameters = {};
            if (schemaEstimatedFee && resourceEstimatedFee) {
                feeParameters = {
                    estimatedTotalTxFee: Number(schemaEstimatedFee.transactionFee) + Number(resourceEstimatedFee.transactionFee),
                    estimatedSchemaTxFee: schemaEstimatedFee,
                    estimatedResourceTxFee: resourceEstimatedFee,
                };
            }
            return feeParameters;
        }
    }
    createDidRegistryInstance(signingKey) {
        if (!this.rpcUrl || !this.didContractAddress) {
            throw new core_1.CredoError('Ledger config not found');
        }
        return new polygon_did_registrar_1.PolygonDID({
            rpcUrl: this.rpcUrl,
            contractAddress: this.didContractAddress,
            signingKey,
        });
    }
    createSchemaRegistryInstance(signingKey) {
        if (!this.rpcUrl ||
            !this.schemaManagerContractAddress ||
            !this.fileServerToken ||
            !this.fileServerUrl ||
            !this.didContractAddress) {
            throw new core_1.CredoError('Polygon schema module config not found');
        }
        return new polygon_schema_manager_1.PolygonSchema({
            rpcUrl: this.rpcUrl,
            didRegistrarContractAddress: this.didContractAddress,
            schemaManagerContractAddress: this.schemaManagerContractAddress,
            fileServerToken: this.fileServerToken,
            serverUrl: this.fileServerUrl,
            signingKey,
        });
    }
    async getSigningKey(wallet, publicKeyBase58) {
        if (!(wallet instanceof askar_1.AskarWallet) && !(wallet instanceof askar_1.AskarProfileWallet)) {
            throw new core_1.CredoError('Incorrect wallet type: Polygon Module currently only supports Askar wallet');
        }
        const keyEntry = await wallet.withSession(async (session) => await session.fetchKey({ name: publicKeyBase58 }));
        if (!keyEntry) {
            throw new core_1.WalletError('Key not found in wallet');
        }
        const signingKey = new ethers_1.SigningKey(keyEntry.key.secretBytes);
        keyEntry.key.handle.free();
        return signingKey;
    }
    async getPublicKeyFromDid(agentContext, did) {
        var _a;
        const didRepository = agentContext.dependencyManager.resolve(core_1.DidRepository);
        const didRecord = await didRepository.findCreatedDid(agentContext, did);
        if (!didRecord) {
            throw new core_1.CredoError('DidRecord not found');
        }
        if (!((_a = didRecord.didDocument) === null || _a === void 0 ? void 0 : _a.verificationMethod)) {
            throw new core_1.CredoError('VerificationMethod not found cannot get public key');
        }
        const publicKeyBase58 = didRecord.didDocument.verificationMethod[0].publicKeyBase58;
        return publicKeyBase58;
    }
    updateModuleConfig({ didRegistrarContractAddress, fileServerToken, rpcUrl, schemaManagerContractAddress, serverUrl, }) {
        this.rpcUrl = rpcUrl;
        this.didContractAddress = didRegistrarContractAddress;
        this.schemaManagerContractAddress = schemaManagerContractAddress;
        this.fileServerToken = fileServerToken;
        this.fileServerUrl = serverUrl;
    }
};
PolygonLedgerService = __decorate([
    (0, core_1.injectable)(),
    __metadata("design:paramtypes", [PolygonModuleConfig_1.PolygonModuleConfig])
], PolygonLedgerService);
exports.PolygonLedgerService = PolygonLedgerService;
//# sourceMappingURL=PolygonLedgerService.js.map