"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSecp256k1DidDocWithPublicKey = exports.generateSecp256k1KeyPair = void 0;
const polygon_did_registrar_1 = require("@ayanworks/polygon-did-registrar");
const core_1 = require("@credo-ts/core");
const signature_suites_1 = require("../signature-suites");
const generateSecp256k1KeyPair = async () => {
    const { privateKey, publicKeyBase58, address } = await polygon_did_registrar_1.PolygonDID.createKeyPair('testnet');
    return { privateKey, publicKeyBase58, address };
};
exports.generateSecp256k1KeyPair = generateSecp256k1KeyPair;
function getSecp256k1DidDocWithPublicKey(did, publicKeyBase58, serviceEndpoint) {
    const verificationMethod = new core_1.VerificationMethod({
        id: `${did}#key-1`,
        type: core_1.VERIFICATION_METHOD_TYPE_ECDSA_SECP256K1_VERIFICATION_KEY_2019,
        controller: did,
        publicKeyBase58,
    });
    const didDocumentBuilder = new core_1.DidDocumentBuilder(did);
    didDocumentBuilder.addContext(signature_suites_1.SECURITY_CONTEXT_SECP256k1_URL).addVerificationMethod(verificationMethod);
    if (serviceEndpoint) {
        const service = new core_1.DidDocumentService({
            id: `${did}#linked-domain`,
            serviceEndpoint,
            type: 'LinkedDomains',
        });
        didDocumentBuilder.addService(service);
    }
    didDocumentBuilder
        .addAuthentication(verificationMethod.id)
        .addAssertionMethod(verificationMethod.id)
        .addCapabilityDelegation(verificationMethod.id)
        .addCapabilityInvocation(verificationMethod.id);
    didDocumentBuilder.addKeyAgreement(verificationMethod.id);
    return didDocumentBuilder.build();
}
exports.getSecp256k1DidDocWithPublicKey = getSecp256k1DidDocWithPublicKey;
//# sourceMappingURL=utils.js.map